use std::{
	env, fs::{File, OpenOptions},
};
use memmap2::MmapOptions;
use rouille::{Response, router};
use btrfs_explorer::main_error::MainError;

fn main() -> Result<(), MainError> {
	let filename = env::args().skip(1).next().ok_or("Argument required")?;

	/*
	let file = OpenOptions::new().read(true).open(filename)?;
	let image = unsafe { Mmap::map(&file)? };
	*/

	let file = OpenOptions::new().read(true).open(filename)?;
	let image = unsafe { MmapOptions::new().len(493921239040usize).map(&file)? };

	rouille::start_server("127.0.0.1:8080", move |request| {
		router!(
			request,
			(GET) ["/"] =>
				btrfs_explorer::http_chunk::http_allchunks(&image, request).unwrap(),
			(GET) ["/root"] =>
				btrfs_explorer::http_tree::http_root(&image, None, request),
			(GET) ["/chunk/{offset}", offset: String] =>
				btrfs_explorer::http_chunk::http_chunk(&image, &offset, request).unwrap(),
			(GET) ["/tree/{tree}", tree: String] =>
				btrfs_explorer::http_tree::http_tree(&image, &tree, None, request.get_param("key").as_deref(), request).unwrap(),
			(GET) ["/tree/{tree}/{key}", tree: String, key: String] =>
				btrfs_explorer::http_tree::http_tree(&image, &tree, None, Some(&key), request).unwrap(),
			(GET) ["/tree/{tree}?key={key}", tree: String, key: String] =>
				btrfs_explorer::http_tree::http_tree(&image, &tree, None, Some(&key), request).unwrap(),
			(GET) ["/tree/{tree}/{method}/{key}", tree: String, method: String, key: String] =>
				btrfs_explorer::http_tree::http_tree(&image, &tree, Some(&method), Some(&key), request).unwrap(),
			(GET) ["/favicon.ico"] => Response::empty_404(),
			(GET) ["/style.css"] => Response::from_file("text/css", File::open("style.css").unwrap()),
			(GET) ["/htmx.min.js"] => Response::from_file("text/css", File::open("htmx.min.js").unwrap()),
			_ => Response::empty_404(),
		)
	});
}
